@@ -50,14 +50,13 @@ enum {
   LAST_SIGNAL
 };
 
-struct _GCancellable
+struct _GCancellablePrivate
 {
-  GObject parent_instance;
-
   guint cancelled : 1;
-  guint allocated_pipe : 1;
   guint cancelled_running : 1;
   guint cancelled_running_waiting : 1;
+
+  guint fd_refcount;
   int cancel_pipe[2];
 
 #ifdef G_OS_WIN32
@@ -73,21 +72,40 @@ static GStaticPrivate current_cancellable = G_STATIC_PRIVATE_INIT;
 G_LOCK_DEFINE_STATIC(cancellable);
 static GCond *cancellable_cond = NULL;
   
+static void
+g_cancellable_close_pipe (GCancellable *cancellable)
+{
+  GCancellablePrivate *priv;
+  
+  priv = cancellable->priv;
+
+  if (priv->cancel_pipe[0] != -1)
+    {
+      close (priv->cancel_pipe[0]);
+      priv->cancel_pipe[0] = -1;
+    }
+  
+  if (priv->cancel_pipe[1] != -1)
+    {
+      close (priv->cancel_pipe[1]);
+      priv->cancel_pipe[1] = -1;
+    }
+
+#ifdef G_OS_WIN32
+  if (priv->event)
+    {
+      CloseHandle (priv->event);
+      priv->event = NULL;
+    }
+#endif
+}
+
 static void
 g_cancellable_finalize (GObject *object)
 {
   GCancellable *cancellable = G_CANCELLABLE (object);
 
-  if (cancellable->cancel_pipe[0] != -1)
-    close (cancellable->cancel_pipe[0]);
-  
-  if (cancellable->cancel_pipe[1] != -1)
-    close (cancellable->cancel_pipe[1]);
-
-#ifdef G_OS_WIN32
-  if (cancellable->event)
-    CloseHandle (cancellable->event);
-#endif
+  g_cancellable_close_pipe (cancellable);
 
   G_OBJECT_CLASS (g_cancellable_parent_class)->finalize (object);
 }
@@ -97,6 +115,8 @@ g_cancellable_class_init (GCancellableClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
+  g_type_class_add_private (klass, sizeof (GCancellablePrivate));
+
   if (cancellable_cond == NULL && g_thread_supported ())
     cancellable_cond = g_cond_new ();
   
@@ -206,26 +226,34 @@ set_fd_close_exec (int fd)
 static void
 g_cancellable_open_pipe (GCancellable *cancellable)
 {
-  if (pipe (cancellable->cancel_pipe) == 0)
+  const char ch = 'x';
+  GCancellablePrivate *priv;
+
+  priv = cancellable->priv;
+  if (pipe (priv->cancel_pipe) == 0)
     {
       /* Make them nonblocking, just to be sure we don't block
        * on errors and stuff
        */
-      set_fd_nonblocking (cancellable->cancel_pipe[0]);
-      set_fd_nonblocking (cancellable->cancel_pipe[1]);
-      set_fd_close_exec (cancellable->cancel_pipe[0]);
-      set_fd_close_exec (cancellable->cancel_pipe[1]);
+      set_fd_nonblocking (priv->cancel_pipe[0]);
+      set_fd_nonblocking (priv->cancel_pipe[1]);
+      set_fd_close_exec (priv->cancel_pipe[0]);
+      set_fd_close_exec (priv->cancel_pipe[1]);
+      
+      if (priv->cancelled)
+        write (priv->cancel_pipe[1], &ch, 1);
     }
-  else
-    g_warning ("Failed to create pipe for GCancellable. Out of file descriptors?");
 }
 #endif
 
 static void
 g_cancellable_init (GCancellable *cancellable)
 {
-  cancellable->cancel_pipe[0] = -1;
-  cancellable->cancel_pipe[1] = -1;
+  cancellable->priv = G_TYPE_INSTANCE_GET_PRIVATE (cancellable,
+					           G_TYPE_CANCELLABLE,
+					           GCancellablePrivate);
+  cancellable->priv->cancel_pipe[0] = -1;
+  cancellable->priv->cancel_pipe[1] = -1;
 }
 
 /**
@@ -323,31 +351,34 @@ g_cancellable_get_current  (void)
 void 
 g_cancellable_reset (GCancellable *cancellable)
 {
+  GCancellablePrivate *priv;
+
   g_return_if_fail (G_IS_CANCELLABLE (cancellable));
 
   G_LOCK(cancellable);
+
+  priv = cancellable->priv;
   
-  while (cancellable->cancelled_running)
+  while (priv->cancelled_running)
     {
-      cancellable->cancelled_running_waiting = TRUE;
+      priv->cancelled_running_waiting = TRUE;
       g_cond_wait (cancellable_cond,
                    g_static_mutex_get_mutex (& G_LOCK_NAME (cancellable)));
     }
   
-  if (cancellable->cancelled)
+  if (priv->cancelled)
     {
       char ch;
       
     /* Make sure we're not leaving old cancel state around */
       
 #ifdef G_OS_WIN32
-      if (cancellable->event)
-	ResetEvent (cancellable->event);
-      else
+      if (priv->event)
+	ResetEvent (priv->event);
 #endif
-      if (cancellable->cancel_pipe[0] != -1)
-	read (cancellable->cancel_pipe[0], &ch, 1);
-      cancellable->cancelled = FALSE;
+      if (priv->cancel_pipe[0] != -1)
+	read (priv->cancel_pipe[0], &ch, 1);
+      priv->cancelled = FALSE;
     }
   G_UNLOCK(cancellable);
 }
@@ -364,7 +395,7 @@ g_cancellable_reset (GCancellable *cancellable)
 gboolean
 g_cancellable_is_cancelled (GCancellable *cancellable)
 {
-  return cancellable != NULL && cancellable->cancelled;
+  return cancellable != NULL && cancellable->priv->cancelled;
 }
 
 /**
@@ -405,6 +436,10 @@ g_cancellable_set_error_if_cancelled (GCancellable  *cancellable,
  * readable status. Reading to unset the readable status is done
  * with g_cancellable_reset().
  * 
+ * After a successful return from this function, you should use 
+ * g_cancellable_release_fd() to free up resources allocated for 
+ * the returned file descriptor.
+ *
  * See also g_cancellable_make_pollfd().
  *
  * Returns: A valid file descriptor. %-1 if the file descriptor 
@@ -413,21 +448,23 @@ g_cancellable_set_error_if_cancelled (GCancellable  *cancellable,
 int
 g_cancellable_get_fd (GCancellable *cancellable)
 {
+  GCancellablePrivate *priv;
   int fd;
+
   if (cancellable == NULL)
     return -1;
 
+  priv = cancellable->priv;
+
 #ifdef G_OS_WIN32
   return -1;
 #else
   G_LOCK(cancellable);
-  if (!cancellable->allocated_pipe)
-    {
-      cancellable->allocated_pipe = TRUE;
-      g_cancellable_open_pipe (cancellable);
-    }
-
-  fd = cancellable->cancel_pipe[0];
+  if (priv->cancel_pipe[0] == -1)
+    g_cancellable_open_pipe (cancellable);
+  fd = priv->cancel_pipe[0];
+  if (fd != -1)
+    priv->fd_refcount++;
   G_UNLOCK(cancellable);
 #endif
 
@@ -436,7 +473,7 @@ g_cancellable_get_fd (GCancellable *cancellable)
 
 /**
  * g_cancellable_make_pollfd:
- * @cancellable: a #GCancellable.
+ * @cancellable: a #GCancellable or %NULL
  * @pollfd: a pointer to a #GPollFD
  * 
  * Creates a #GPollFD corresponding to @cancellable; this can be passed
@@ -444,29 +481,100 @@ g_cancellable_get_fd (GCancellable *cancellable)
  * for unix systems without a native poll and for portability to
  * windows.
  *
+ * When this function returns %TRUE, you should use 
+ * g_cancellable_release_fd() to free up resources allocated for the 
+ * @pollfd. After a %FALSE return, do not call g_cancellable_release_fd().
+ *
+ * If this function returns %FALSE, either no @cancellable was given or
+ * resource limits prevent this function from allocating the necessary 
+ * structures for polling. (On Linux, you will likely have reached 
+ * the maximum number of file descriptors.) The suggested way to handle
+ * these cases is to ignore the @cancellable.
+ *
  * You are not supposed to read from the fd yourself, just check for
  * readable status. Reading to unset the readable status is done
  * with g_cancellable_reset().
+ *
+ * @Returns: %TRUE if @pollfd was successfully initialized, %FALSE on 
+ *           failure to prepare the cancellable.
  * 
+ * @Since: 2.22
  **/
-void
+gboolean
 g_cancellable_make_pollfd (GCancellable *cancellable, GPollFD *pollfd)
 {
-  g_return_if_fail (G_IS_CANCELLABLE (cancellable));
-  g_return_if_fail (pollfd != NULL);
+  GCancellablePrivate *priv;
+
+  g_return_val_if_fail (pollfd != NULL, FALSE);
+  if (cancellable == NULL)
+    return FALSE;
+  g_return_val_if_fail (G_IS_CANCELLABLE (cancellable), FALSE);
+
+  priv = cancellable->priv;
 
 #ifdef G_OS_WIN32
-  if (!cancellable->event)
+  G_LOCK(cancellable);
+  if (priv->event == NULL)
     {
       /* A manual reset anonymous event, starting unset */
-      cancellable->event = CreateEvent (NULL, TRUE, FALSE, NULL);
+      priv->event = CreateEvent (NULL, TRUE, FALSE, NULL);
+      if (priv->event == NULL)
+        {
+          G_UNLOCK(cancellable);
+          return FALSE;
+        }
+      if (priv->cancelled)
+        SetEvent(priv->event);
     }
-  pollfd->fd = (gintptr)cancellable->event;
+  priv->fd_refcount++;
+  G_UNLOCK(cancellable);
+
+  pollfd->fd = (gintptr)priv->event;
 #else /* !G_OS_WIN32 */
-  pollfd->fd = g_cancellable_get_fd (cancellable);
+  {
+    int fd = g_cancellable_get_fd (cancellable);
+    if (fd == -1)
+      return FALSE;
+    pollfd->fd = fd;
+  }
 #endif /* G_OS_WIN32 */
   pollfd->events = G_IO_IN;
   pollfd->revents = 0;
+
+  return TRUE;
+}
+
+/**
+ * g_cancellable_release_fd:
+ * @cancellable: a #GCancellable
+ *
+ * Releases a resources previously allocated by g_cancellable_get_fd()
+ * or g_cancellable_make_pollfd().
+ *
+ * For compatibility reasons with older releases, calling this function 
+ * is not strictly required, the resources will be automatically freed
+ * when the @cancellable is finalized. However, the @cancellable will
+ * block scarce file descriptors until it is finalized if this function
+ * is not called. This can cause the application to run out of file 
+ * descriptors when many #GCancellables are used at the same time.
+ * 
+ * @Since: 2.22
+ **/
+void
+g_cancellable_release_fd (GCancellable *cancellable)
+{
+  GCancellablePrivate *priv;
+
+  g_return_if_fail (G_IS_CANCELLABLE (cancellable));
+  g_return_if_fail (cancellable->priv->fd_refcount > 0);
+
+  priv = cancellable->priv;
+
+  G_LOCK (cancellable);
+  priv->fd_refcount--;
+  if (priv->fd_refcount == 0)
+    g_cancellable_close_pipe (cancellable);
+  G_UNLOCK (cancellable);
 }
 
 /**
@@ -491,25 +599,27 @@ g_cancellable_make_pollfd (GCancellable *cancellable, GPollFD *pollfd)
 void
 g_cancellable_cancel (GCancellable *cancellable)
 {
+  const char ch = 'x';
   gboolean cancel;
+  GCancellablePrivate *priv;
 
+  if (cancellable == NULL ||
+      cancellable->priv->cancelled)
+    return;
+
+  priv = cancellable->priv;
   cancel = FALSE;
 
   G_LOCK(cancellable);
-  if (cancellable != NULL &&
-      !cancellable->cancelled)
-    {
-      char ch = 'x';
-      cancel = TRUE;
-      cancellable->cancelled = TRUE;
-      cancellable->cancelled_running = TRUE;
+  cancel = TRUE;
+  priv->cancelled = TRUE;
+  priv->cancelled_running = TRUE;
 #ifdef G_OS_WIN32
-      if (cancellable->event)
-	SetEvent(cancellable->event);
+  if (priv->event)
+    SetEvent (priv->event);
 #endif
-      if (cancellable->cancel_pipe[1] != -1)
-	write (cancellable->cancel_pipe[1], &ch, 1);
-    }
+  if (priv->cancel_pipe[1] != -1)
+    write (priv->cancel_pipe[1], &ch, 1);
   G_UNLOCK(cancellable);
 
   if (cancel)
@@ -519,10 +629,10 @@ g_cancellable_cancel (GCancellable *cancellable)
 
       G_LOCK(cancellable);
 
-      cancellable->cancelled_running = FALSE;
-      if (cancellable->cancelled_running_waiting)
+      priv->cancelled_running = FALSE;
+      if (priv->cancelled_running_waiting)
 	g_cond_broadcast (cancellable_cond);
-      cancellable->cancelled_running_waiting = FALSE;
+      priv->cancelled_running_waiting = FALSE;
 
       G_UNLOCK(cancellable);
 
@@ -568,7 +678,7 @@ g_cancellable_connect (GCancellable   *cancellable,
 
   G_LOCK (cancellable);
 
-  if (cancellable->cancelled)
+  if (cancellable->priv->cancelled)
     {
       void (*_callback) (GCancellable *cancellable,
                          gpointer      user_data);
@@ -616,13 +726,18 @@ void
 g_cancellable_disconnect (GCancellable  *cancellable,
 			  gulong         handler_id)
 {
+  GCancellablePrivate *priv;
+
   if (handler_id == 0 ||  cancellable == NULL)
     return;
 
   G_LOCK (cancellable);
-  while (cancellable->cancelled_running)
+
+  priv = cancellable->priv;
+
+  while (priv->cancelled_running)
     {
-      cancellable->cancelled_running_waiting = TRUE;
+      priv->cancelled_running_waiting = TRUE;
       g_cond_wait (cancellable_cond,
                    g_static_mutex_get_mutex (& G_LOCK_NAME (cancellable)));
     }
