@@ -14,7 +14,7 @@ for so in .libs/lib*.so; do
 	# g_string_insert_c is used in g_string_append_c_inline
 	# unaliased.  Couldn't find a way to fix it.
 	# Same for g_once_init_enter
-	readelf -r $so | grep 'JU\?MP_SLOT' | \
+	readelf -r $so | grep 'JU\?MP_SLOT\?' | \
 		grep -v '\<g_string_insert_c\>' | \
 		grep -v '\<g_atomic_[a-z]*_[sg]et\>' | \
 		grep -v '\<g_once_init_enter_impl\>' | \
