@@ -263,10 +263,47 @@ GType
 g_regex_get_type (void)
 {
   static GType type_id = 0;
+
+#ifdef ENABLE_REGEX
   if (!type_id)
     type_id = g_boxed_type_register_static (g_intern_static_string ("GRegex"),
 					    (GBoxedCopyFunc) g_regex_ref,
 					    (GBoxedFreeFunc) g_regex_unref);
+#endif
+
+  return type_id;
+}
+
+GType
+g_array_get_type (void)
+{
+  static GType type_id = 0;
+  if (!type_id)
+    type_id = g_boxed_type_register_static (g_intern_static_string ("GArray"),
+					    (GBoxedCopyFunc) g_array_ref,
+                                            (GBoxedFreeFunc) g_array_unref);
+  return type_id;
+}
+
+GType
+g_ptr_array_get_type (void)
+{
+  static GType type_id = 0;
+  if (!type_id)
+    type_id = g_boxed_type_register_static (g_intern_static_string ("GPtrArray"),
+					    (GBoxedCopyFunc) g_ptr_array_ref,
+                                            (GBoxedFreeFunc) g_ptr_array_unref);
+  return type_id;
+}
+
+GType
+g_byte_array_get_type (void)
+{
+  static GType type_id = 0;
+  if (!type_id)
+    type_id = g_boxed_type_register_static (g_intern_static_string ("GByteArray"),
+					    (GBoxedCopyFunc) g_byte_array_ref,
+                                            (GBoxedFreeFunc) g_byte_array_unref);
   return type_id;
 }
 
